//
// DESCRIPTION:
//	DOOM graphics stuff for VoCore Display, OpenWrt.
//  based on i_video.c.
//
//-----------------------------------------------------------------------------

#include "includes.h"

#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/mman.h>

unsigned int SCREENWIDTH = 320;
unsigned int SCREENHEIGHT = 240;
unsigned int ORIGSCREENWIDTH = 320;
unsigned int ORIGSCREENHEIGHT = 200;

static unsigned short palette_table[256];
static byte *framebuffer;

void I_InitGraphics (void)
{
	int fd = open("/dev/fb0", O_RDWR);
	if (fd < 0) {
		printf("can not open framebuffer.\n");
		exit(-1);
	}

	framebuffer = mmap(NULL, 800 * 480 * 2, PROT_READ|PROT_WRITE,
			   MAP_SHARED, fd, 0);
	if (framebuffer == NULL) {
		printf("can not mmap framebuffer.\n");
		exit(-1);
	}

	system("dd if=/dev/zero of=/dev/fb0 count=1 bs=768000 2> /dev/null");
}

void I_ShutdownGraphics(void)
{
}

void swap_byte(unsigned short *in)
{
	char *p = (char *)in, tmp;
	tmp = p[0];
	p[0] = p[1];
	p[1] = tmp;
}

void I_SetPalette (byte* palette)
{
    unsigned int colour = 0;
    do {
	palette_table[colour] = 0;

#ifdef USE_SCREEN_VOCORE2
	palette_table[colour] |= (((unsigned short)gammatable[usegamma][*palette++]) >> 3);
	palette_table[colour] |= (((unsigned short)gammatable[usegamma][*palette++]) >> 2) << 5;
	palette_table[colour] |= (((unsigned short)gammatable[usegamma][*palette++]) >> 3) << 11;
	swap_byte(palette_table + colour);
#endif

#ifdef USE_SCREEN_DJN1522
	palette_table[colour] |= (((unsigned short)gammatable[usegamma][*palette++]) >> 3) << 11;
	palette_table[colour] |= (((unsigned short)gammatable[usegamma][*palette++]) >> 2) << 5;
	palette_table[colour] |= (((unsigned short)gammatable[usegamma][*palette++]) >> 3);
#endif
    } while (++colour < 256);
}

void show_fps()
{
    static time_t last;
    static int fps;

    time_t now = time(NULL);
    if (last == now)
	fps++;
    else {
	last = now;
	printf("fps: %d\n", fps);
	fps = 0;
    }
}

void I_FinishUpdate (void)
{
    byte *src = screens[0], *dst = framebuffer;
    int i, j, pos;

#ifndef SMALL_SCREEN
    int w, h;
    h = SCREENHEIGHT * 4;
    w = SCREENWIDTH * 4;

    for (j = 0; j < h; j += 4) {
	for (i = 0; i < w; i += 4) {
	    pos = 720 * 480 * 2 - i * 480 + j;
	    *(unsigned short *)(dst + pos    ) = palette_table[*src];
	    *(unsigned short *)(dst + pos + 2) = palette_table[*src];
	    *(unsigned short *)(dst + pos - 960) = palette_table[*src];
	    *(unsigned short *)(dst + pos - 958) = palette_table[*src];
	    src++;
	}
    }
#else
    for (j = 0; j < SCREENHEIGHT; j++) {
	pos = j * 480 * 2;
	for (i = 0; i < SCREENWIDTH; i++)
	    *(unsigned short *)(dst + pos + i * 2) = palette_table[*src++];
    }
#endif
    usleep(16000);
    show_fps();
}

void I_ReadScreen (byte* scr)
{
    memcpy(scr, screens[0], SCREENWIDTH * SCREENHEIGHT);
}

void I_InitKeyboard (void)
{
}

void I_UpdateNoBlit (void)
{
}

void I_SetScreenSize (void)
{
}

void I_SetWindowName (const char * title)
{
}

void I_StartFrame (void)
{
}

void I_StartTic (void)
{
}

